# copyright erik osheim, 2011
# released to the public domain

import random
import sys

class Namer(object):
    vowels = set('aeiou')
    def __init__(self):
        self.probs = {}
        self.weights = {}

    def learnfile(self, path):
        names = [s.strip() for s in open(path).readlines()]
        self.learn(names)

    def learn(self, names):
        for name in names:
            assert len(name) > 1
            s = '^%' + name + '$'
            for i in range(0, len(s) - 2):
                c1, c2, c3 = s[i], s[i + 1], s[i + 2]
                key = (c1, c2)
                val = c3
                self.probs.setdefault(key, {})
                self.probs[key].setdefault(val, 0)
                self.probs[key][val] += 1

                self.weights.setdefault(key, 0)
                self.weights[key] += 1

    def getchar(self, key):
        vals = list(self.probs[key])
        weight = self.weights[key]
        n = random.randint(0, weight - 1)
        t = 0
        i = 0
        while t < n and i < len(vals) - 1:
            t += self.probs[key][vals[i]]
            i += 1
        return vals[i]

    def generate(self, minlen=3, maxlen=10, minvowels=1):
        vowels = 0
        s = None
        while s is None or len(s) < minlen or vowels < minvowels:
            vowels = 0
            s = self._generate(10)
            for c in s:
                if c in self.vowels:
                    vowels += 1
        return s

    def _generate(self, maxlen):
        chars = []
        c1, c2 = '^', '%'

        c3 = self.getchar((c1, c2))
        i = 0
        while c3 != '$' and i < maxlen:
            chars.append(c3)
            c1, c2 = c2, c3
            c3 = self.getchar((c1, c2))
            i += 1

        return ''.join(chars)

if __name__ == "__main__":
    namer = Namer()
    namer.learnfile('base.txt')

    for i in range(0, 10):
        print namer.generate().title()
